# sb::Session
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == Copyright (C) 2005 SimpleBoxes/SerendipityNZ Ltd. ==

# ======================================
package sb::Session;
use strict;
use sb ();
use sb::Interface ();
use sb::Data ();
use sb::Object ();
use base qw( sb::Object );
use vars qw( $VERSION );
$VERSION = '0.07';
# 0.07 [2009/05/27] modified id to generate random strings
# 0.06 [2007/08/15] changed error string
# 0.05 [2007/08/04] changed initialize
# 0.04 [2007/07/20] changed id to call sb->pid() instead of sb->get->pid()
# 0.03 [2007/07/16] changed return to make them consistent
# 0.02 [2006/07/04] got rid of "return( undef )"
# 0.01 [2006/03/17] changed bracket rule
# 0.00 [2006/03/04] refactored
# --------------------------------------
# // defines / static variables
sub DEFAULT_EXPIRES (){ 1 };
sub COOKIENAME      (){ 'sb_session' };
sub HOURS_PER_DAY   (){ 24 };
sub SECS_PER_HOUR   (){ 3600 };
# --------------------------------------
# // override base class
sub initialize
{
	my $self = shift;
	$self->SUPER::initialize(
		'key'    => undef,           # [required][CHAR] key for session
		'cgi'    => undef,           # [optional][OBJ.] sb::Interface object
		'time'   => undef,           # [optional][NUM.] current time (UnixTime on GMT)
		'expire' => DEFAULT_EXPIRES, # [optional][NUM.] days for expires
		'name'   => COOKIENAME,      # [optional][CHAR] name for cookie
		'path'   => undef,           # [optional][URI.] path for cookie
		'domain' => undef,           # [optional][URI.] domain for cookie
		@_
	);
	$self->{'cgi'} = sb::Interface->get if (!$self->{'cgi'});
	$self->{'id'} = undef;
	$self->{'stored'} = undef;
	return $self->_initialize();
}
# --------------------------------------
# // public functions
sub id
{ # create session unique id
	my $self = shift;
	if (! defined($self->{'id'}))
	{
		my @seeds = ('a'..'z','A'..'Z','0'..'9');
		$self->{'id'}  = $self->{'time'} . '_' . sb->pid() . '_';
		$self->{'id'} .= join('',map { $seeds[ rand @seeds ] } 0 .. 20);
	}
	return $self->{'id'};
}
sub check
{ # check session
	my $self = shift;
	return (defined($self->{'stored'}));
}
sub start
{ # start session
	my $self = shift;
	$self->_set_data($self->id);
	$self->{'cgi'}->set_cookie(
		'time'   => $self->{'time'},
		'name'   => $self->{'name'},
		'expire' => $self->{'expire'},
		'path'   => $self->{'path'},
		'domain' => $self->{'domain'},
		'data'   => {'id' => $self->id},
	);
	return;
}
sub finish
{ # finish session
	my $self = shift;
	$self->_set_data(0);
}
# --------------------------------------
# // private functions
sub _initialize
{
	my $self = shift;
	if (defined($self->{'key'}))
	{
		$self->{'time'} ||= time(); # set the current time
		my $check_id = $self->_check_cookie;
		my $update_flag = undef;
		my @sessions = sb::Data->load('Session','cond'=>{'key'=>$self->{'key'}});
		foreach my $session ( @sessions )
		{
			if ($check_id and $session->data eq $check_id)
			{
				$self->{'stored'} = $session;
			}
			if (!$session->check_expire('now'=>$self->{'time'},'duration'=>$self->_expire_time))
			{
				$session->erase;
				$update_flag = 1;
			}
		}
		sb::Data->update(@sessions) if ($update_flag);
	}
	else
	{
		die("sb::Session : Need to set key\n");
	}
	return $self;
}
sub _expire_time
{ # calculate expire time [sec]
	my $self = shift;
	return $self->{'expire'} * HOURS_PER_DAY * SECS_PER_HOUR;
}
sub _check_cookie
{ # read cookie data
	my $self = shift;
	my $cookie = $self->{'cgi'}->cookie('name'=>$self->{'name'});
	if ( $cookie->{'id'} )
	{
		my $diff_time = $self->{'time'} - ( split('_',$cookie->{'id'}) )[0];
		return if ( $diff_time > $self->_expire_time );
	}
	return $cookie->{'id'};
}
sub _set_data
{ # update sb::Data::Session
	my $self = shift;
	my $data = shift;
	my $session = ( $self->{'stored'} ) 
		? $self->{'stored'} 
		: sb::Data->add('Session','key'=>$self->{'key'});
	die('Failed to create session') if (!$session);
	$session->data($data); # set data
	sb::Data->update($session);
}
1;
=head1 NAME

sb::Session - session management

=head1 SYNOPSIS

	use sb::Session;
	my $session = sb::Session->new('key' => 'session_key');
	if ( $session->check )
	{
		# session is valid
		# something to do ...
		$session->finish; # finishing session
	}
	else
	{
		# session is invalid
		# something to do, such as user authorization
		$session->start; # starting session
	}

=head1 DESCRIPTION

sb::Session requires sb::Data::Session.
Session data is stored as sb::Data::Object.
Need to set 'key' to create session.
'key' indicates each session, so sb::Session allows multiple sessions.

=head1 AUTHOR

Takuya Otani <takuya.otani@gmail.com> 

=head1 COPYRIGHT & LICENSE

Copyright (C) 2005 Takuya Otani@SimpleBoxes / SerendipityNZ Ltd.

=cut
__END__
